/*
 Comprueba si dos cadenas tienen el mismo numero de caracteres 
 de cada carácter que tienen en la cadena. No se consideran
 caracteres que no sean 'A'-'Z' o 'a'-'z', y cuentan como
 iguales mayúsculas y minúsculas

 Para ello:

  - Creamos una tabla para cada cadena con la cuenta
    de cada carácter en la posición de dicho carácter:

       num. de caracteres tipo i = tabla[i]

    con a = carácter 0 y z = carácter 25

  - Comparamos si ambas tablas son iguales.

*/
      .data
cad1: .asciz "AbcDeFg  ZZZ.......ZZZ ., AbCdEFF AA,,,,,,,"
cad2: .asciz "AbcDeFg  ZZZZCdEFF AAZZ ., Ab"
tab1: .space 26
tab2: .space 26

      .text
main: ldr     r0, =cad1
      ldr     r1, =tab1
      bl      tabcar

      ldr     r0, =cad2
      ldr     r1, =tab2
      bl      tabcar

      ldr     r0, =tab1
      ldr     r1, =tab2
      mov     r2, #26
      bl      compvec

      wfi
/*
 tabcar: 
 Subprograma para contar el numero de ejemplares de cada letra 
 que tiene una cadena.

      entrada: r0: dirección de la cadena terminada en 0 
               r1: dirección de la tabla para el resultado
      salida: tabla actualizada.
*/

      .equ      mascara,  0b11011111    @ para pasar a mayúsculas
      .equ      lpost, 25               @ la tabla tiene 26 slots
                                        @ (de 0 a 25)
tabcar:      
@
@ Inicializamos la tabla a cero
@
      mov      r2, #0
      mov      r3, #lpost
bini:      
      strb     r2, [r1, r3]
      sub      r3, r3, #1
      cmp      r3, #0
      bne      bini
      strb     r2, [r1]
@
@ contamos el numero de apariciones de cada letra que aparece
@
      mov      r3, #mascara  @ para pasar a mayúsculas
bucles:      
      ldrb     r2, [r0]      @ cargamos un carácter
      cmp      r2, #0
      beq      terms
      
      and      r2, r2, r3    @ pasamos todo a mayúsculas y
      cmp      r2, #'A'      @ solo consideramos letras
      blt      noletra

      cmp      r2, #'Z'
      bgt      noletra
@
@ si llegamos aquí tenemos letras
@
      sub      r2, #'A'      @ dejamos las letras en el rango 0-25
      ldrb     r3, [r1, r2]  @ y lo utilizamos de índice en la tabla
      add      r3, r3, #1    @ para actualizar la cuenta del "slot"
      strb     r3, [r1, r2]  @ correspondiente
noletra:
      add      r0, r0, #1    @ actualizamos el puntero a la cadena
      b        bucles

terms:
      mov      pc, lr
@
@ Fin de tabcar
@
/*
 compvec : subprograma para comparar dos vectores del mismo tamaño

      entrada: r0, dir. del primer vector; 
      r1 = dir. del segundo vector, 
      r2 = tamaño de los vectores
      salida: r0 = 1 si son iguales, 0 en caso contrario. 
 
*/
compvec:
      push     {r4, lr}      @ preservamos r4, y de paso lr

bucc: cmp      r2, #0        @ vemos si hemos terminado
      beq      igual

      sub      r2, r2, #1    @ movemos el índice
      ldrb     r3, [r0, r2]  @ cargamos valores
      ldrb     r4, [r1, r2]
      cmp      r3, r4        @ y comparamos
      bne      dist
      b        bucc          @ el siguiente

igual:      
      mov      r0, #1        @ indicador a "son iguales"
      b        fsr
dist:      
      mov      r0, #0        @ indicador a "son distintos"
fsr:  pop      {r4, pc}      @ recuperamos r4, y de paso pc
      .end