@
@ Ejemplo extraído del ejercicio indicado
@

       .data
array: .byte 0b10000000,0b00000000 @ fila 0
       .byte 0b01000000,0b00000000 @ fila 1
       .byte 0b00100000,0b00000000 @ fila 2
       .byte 0b00001000,0b11111101 @ fila 3
@              ||||||||   ||||||||       
@ columnas:    01234567   89abcdef
@
      .balign 4
row:  .word 2      @ fila de ejemplo
col:  .word 12     @ columna de ejemplo

      .text

@
@ definimos códigos de error
@
.equ  nif,1        @ error de numero de filas
.equ  nic,2        @ error de numero de columnas
.equ  ffr,3        @ error de fila fuera de rango
.equ  cfr,4        @ error de columna fuera de rango

@
@ Programa principal
@ 
@ Parámetros: activar (1) el bit de coordenadas (2, 12)
@ de una matriz 4 x 16:
@  
@     - r0 = 4
@     - r1 = 16
@     - r2 = 2   [row]
@     - r3 = 12  [col]
@
main: mov r0,#4     @ 4 filas 
      mov r1,#16    @ 16 columnas
      ldr r2,=row   @ fila a activar
      ldr r2,[r2]
      ldr r3,=col   @ columna a activar
      ldr r3,[r3]
      ldr r5,=array @ dirección de la matriz
      mov r4,#1     @ activar

      push {r4,r5}  @ estos parámetros se pasan en la pila
      bl  setbit
      add  sp,#8    @ balanceamos la pila

      wfi

/*
 setbit: subrutina para activar o desactivar 
   el bit en la posición (i,j) en una matriz de 
   tamaño (m x n) serializada por filas y almacenada
   a partir de la dirección dirmat.
 
   Parámetros:
    r0: m (número de filas de la matriz)
    r1: n (número de columnas de la matriz)
    r2: i (fila del bit a activar, de 0 a m-1)
    r3: j (columna del bit a activar, de 0 a n-1)
    En la pila (dos palabras, apiladas en el orden indicado: 
       - dirmat (dirección de la matiz).
       - 1 para indicar activación y 0 para indicar desactivación.
 
   Resultado: r0: 1 si el bit se activó, y 0 si hubo error
              r1: código de error si hubo error, 0 en caso contrario
 */
setbit:
@
@ Comprobamos errores en los parámetros de entrada
@
      cmp r0,#0
      ble e_nif         @ error de numero de filas (M <= 0)
      cmp r1,#0
      ble e_nic         @ error de número de columnas (N <= 0)
      cmp r2,r0
      bge e_ffr         @ error de fila fuera de rango (i >= M)
      cmp r3,r1
      bge e_cfr         @ error de columna fuera de rango (j >= N)
      cmp r2,#0
      blt e_ffr         @ error de fila fuera de rango (f < 0)
      cmp r3,#0
      blt e_cfr         @ error de columna fuera de rango (c < 0)      
@
@ Calculamos el byte y el offset en ese byte
@ donde está el bit a activar. Primero calculamos el índice
@ absoluto del bit en la matriz serializada y lo dejamos en r2:
@
@ index = i * N + j (valor entre 0 y (N * M) - 1)
@
      mul r2,r2,r1      @ r2: i * N
      add r2,r2,r3      @ r2: index 
@
@ ahora calculamos byte y offset a partir de index (a r0 y r1)
@
@ r0: byte = index div 8; r1: offset: index mod 8

      mov r1,#0x07      @ máscara para calcular el resto
      and r1,r1,r2      @ resto a r1
      lsr r0,r2,#3      @ cociente a r0
@
@ activamos o desactivamos el bit de destino (bit r1 del byte
@ en [dirmat + r0]). Utilizamos como máscara de partida
@ 0x80, y desplazamos a la derecha el offset almacenado en r1
@
      ldr r2,=#0x80     @ máscara de partida para cambiar 
      lsr r2,r1         @ máscara final (con el bit en offset)    
      ldr r3,[sp,#4]    @ dirección de la matriz
      add r3,r3,r0      @ r3: dir del byte a modificar
      ldrb r1,[r3]      @ r1: byte a modificar
      ldr r0,[sp]       @ lsb de r0: activar o desactivar
 
      cmp r0,#0         @ ver si activar o desactivar
      beq set0      

set1: orr r1,r2         @ activa el bit objetivo
      b sigue

set0: mvn r2,r2
      and r1,r2         @ desactiva bir objetivo

sigue: 
      strb r1,[r3]      @ sustituye el byte
      mov r0,#1         @ original con el bit objetivo
      b back            @ y salimos sin errores
@
@ resultados con errores
@
e_nif:
      mov r1,#nif       @ número de filas incorrecto
      b   backe
e_nic:
      mov r1,#nic       @ número de columnas incorrecto
      b   backe
e_ffr:
      mov r1,#ffr       @ fila fuera de rango
      b   backe
e_cfr:
      mov r1,#cfr       @ fila fuera de rango
backe:
      mov r0,#0
back: mov pc,lr         @ volvemos
      .end
